@@ -6,8 +6,8 @@ of the public API.
 The classes should behave roughly in the style of scikit-learn.
 
 - All data-independent parameters should be passed to the class constructor.
-- Each class should impelment a default transformation that is exposed through
-  __call__. These are currently written for vector arguements, but I think
+- Each class should implement a default transformation that is exposed through
+  __call__. These are currently written for vector arguments, but I think
   consuming a whole `plot_data` DataFrame and return it with transformed
   variables would make more sense.
 - Some class have data-dependent preprocessing that should be cached and used
@@ -24,12 +24,17 @@ The classes should behave roughly in the style of scikit-learn.
   class instantiation.
 
 """
-from distutils.version import LooseVersion
 from numbers import Number
 import numpy as np
-import scipy as sp
-from scipy import stats
+import pandas as pd
+try:
+    from scipy.stats import gaussian_kde
+    _no_scipy = False
+except ImportError:
+    from .external.kde import gaussian_kde
+    _no_scipy = True
 
+from .algorithms import bootstrap
 from .utils import _check_argument
 
 
@@ -60,10 +65,10 @@ class KDE:
             Factor, multiplied by the smoothing bandwidth, that determines how
             far the evaluation grid extends past the extreme datapoints. When
             set to 0, truncate the curve at the data limits.
-        clip : pair of numbers None, or a pair of such pairs
+        clip : pair of numbers or None, or a pair of such pairs
             Do not evaluate the density outside of these limits.
         cumulative : bool, optional
-            If True, estimate a cumulative distribution function.
+            If True, estimate a cumulative distribution function. Requires scipy.
 
         """
         if clip is None:
@@ -76,6 +81,9 @@ class KDE:
         self.clip = clip
         self.cumulative = cumulative
 
+        if cumulative and _no_scipy:
+            raise RuntimeError("Cumulative KDE evaluation requires scipy")
+
         self.support = None
 
     def _define_support_grid(self, x, bw, cut, clip, gridsize):
@@ -129,12 +137,9 @@ class KDE:
         """Fit the scipy kde while adding bw_adjust logic and version check."""
         fit_kws = {"bw_method": self.bw_method}
         if weights is not None:
-            if LooseVersion(sp.__version__) < "1.2.0":
-                msg = "Weighted KDE requires scipy >= 1.2.0"
-                raise RuntimeError(msg)
             fit_kws["weights"] = weights
 
-        kde = stats.gaussian_kde(fit_data, **fit_kws)
+        kde = gaussian_kde(fit_data, **fit_kws)
         kde.set_bandwidth(kde.factor * self.bw_adjust)
 
         return kde
@@ -204,13 +209,14 @@ class Histogram:
 
         Parameters
         ----------
-        stat : {"count", "frequency", "density", "probability"}
+        stat : str
             Aggregate statistic to compute in each bin.
 
-            - ``count`` shows the number of observations
-            - ``frequency`` shows the number of observations divided by the bin width
-            - ``density`` normalizes counts so that the area of the histogram is 1
-            - ``probability`` normalizes counts so that the sum of the bar heights is 1
+            - `count`: show the number of observations in each bin
+            - `frequency`: show the number of observations divided by the bin width
+            - `probability` or `proportion`: normalize such that bar heights sum to 1
+            - `percent`: normalize such that bar heights sum to 100
+            - `density`: normalize such that the total area of the histogram equals 1
 
         bins : str, number, vector, or a pair of such values
             Generic bin parameter that can be the name of a reference rule,
@@ -229,7 +235,10 @@ class Histogram:
             If True, return the cumulative statistic.
 
         """
-        _check_argument("stat", ["count", "density", "probability", "frequency"], stat)
+        stat_choices = [
+            "count", "frequency", "density", "probability", "proportion", "percent",
+        ]
+        _check_argument("stat", stat_choices, stat)
 
         self.stat = stat
         self.bins = bins
@@ -238,7 +247,7 @@ class Histogram:
         self.discrete = discrete
         self.cumulative = cumulative
 
-        self.bin_edges = None
+        self.bin_kws = None
 
     def _define_bin_edges(self, x, weights, bins, binwidth, binrange, discrete):
         """Inner function that takes bin parameters as arguments."""
@@ -252,20 +261,30 @@ class Histogram:
         elif binwidth is not None:
             step = binwidth
             bin_edges = np.arange(start, stop + step, step)
+            # Handle roundoff error (maybe there is a less clumsy way?)
+            if bin_edges.max() < stop or len(bin_edges) < 2:
+                bin_edges = np.append(bin_edges, bin_edges.max() + step)
         else:
             bin_edges = np.histogram_bin_edges(
                 x, bins, binrange, weights,
             )
         return bin_edges
 
-    def define_bin_edges(self, x1, x2=None, weights=None, cache=True):
-        """Given data, return the edges of the histogram bins."""
+    def define_bin_params(self, x1, x2=None, weights=None, cache=True):
+        """Given data, return numpy.histogram parameters to define bins."""
         if x2 is None:
 
             bin_edges = self._define_bin_edges(
                 x1, weights, self.bins, self.binwidth, self.binrange, self.discrete,
             )
 
+            if isinstance(self.bins, (str, Number)):
+                n_bins = len(bin_edges) - 1
+                bin_range = bin_edges.min(), bin_edges.max()
+                bin_kws = dict(bins=n_bins, range=bin_range)
+            else:
+                bin_kws = dict(bins=bin_edges)
+
         else:
 
             bin_edges = []
@@ -304,23 +323,23 @@ class Histogram:
                     x, weights, bins, binwidth, binrange, discrete,
                 ))
 
-            bin_edges = tuple(bin_edges)
+            bin_kws = dict(bins=tuple(bin_edges))
 
         if cache:
-            self.bin_edges = bin_edges
+            self.bin_kws = bin_kws
 
-        return bin_edges
+        return bin_kws
 
     def _eval_bivariate(self, x1, x2, weights):
         """Inner function for histogram of two variables."""
-        bin_edges = self.bin_edges
-        if bin_edges is None:
-            bin_edges = self.define_bin_edges(x1, x2, cache=False)
+        bin_kws = self.bin_kws
+        if bin_kws is None:
+            bin_kws = self.define_bin_params(x1, x2, cache=False)
 
         density = self.stat == "density"
 
-        hist, _, _ = np.histogram2d(
-            x1, x2, bin_edges, weights=weights, density=density
+        hist, *bin_edges = np.histogram2d(
+            x1, x2, **bin_kws, weights=weights, density=density
         )
 
         area = np.outer(
@@ -328,8 +347,10 @@ class Histogram:
             np.diff(bin_edges[1]),
         )
 
-        if self.stat == "probability":
+        if self.stat == "probability" or self.stat == "proportion":
             hist = hist.astype(float) / hist.sum()
+        elif self.stat == "percent":
+            hist = hist.astype(float) / hist.sum() * 100
         elif self.stat == "frequency":
             hist = hist.astype(float) / area
 
@@ -343,17 +364,19 @@ class Histogram:
 
     def _eval_univariate(self, x, weights):
         """Inner function for histogram of one variable."""
-        bin_edges = self.bin_edges
-        if bin_edges is None:
-            bin_edges = self.define_bin_edges(x, weights=weights, cache=False)
+        bin_kws = self.bin_kws
+        if bin_kws is None:
+            bin_kws = self.define_bin_params(x, weights=weights, cache=False)
 
         density = self.stat == "density"
-        hist, _ = np.histogram(
-            x, bin_edges, weights=weights, density=density,
+        hist, bin_edges = np.histogram(
+            x, **bin_kws, weights=weights, density=density,
         )
 
-        if self.stat == "probability":
+        if self.stat == "probability" or self.stat == "proportion":
             hist = hist.astype(float) / hist.sum()
+        elif self.stat == "percent":
+            hist = hist.astype(float) / hist.sum() * 100
         elif self.stat == "frequency":
             hist = hist.astype(float) / np.diff(bin_edges)
 
@@ -366,7 +389,7 @@ class Histogram:
         return hist, bin_edges
 
     def __call__(self, x1, x2=None, weights=None):
-        """Count the occurrances in each bin, maybe normalize."""
+        """Count the occurrences in each bin, maybe normalize."""
         if x2 is None:
             return self._eval_univariate(x1, weights)
         else:
@@ -376,7 +399,7 @@ class Histogram:
 class ECDF:
     """Univariate empirical cumulative distribution estimator."""
     def __init__(self, stat="proportion", complementary=False):
-        """Initialize the class with its paramters
+        """Initialize the class with its parameters
 
         Parameters
         ----------
@@ -424,3 +447,106 @@ class ECDF:
             return self._eval_univariate(x1, weights)
         else:
             return self._eval_bivariate(x1, x2, weights)
+
+
+class EstimateAggregator:
+
+    def __init__(self, estimator, errorbar=None, **boot_kws):
+        """
+        Data aggregator that produces an estimate and error bar interval.
+
+        Parameters
+        ----------
+        estimator : callable or string
+            Function (or method name) that maps a vector to a scalar.
+        errorbar : string, (string, number) tuple, or callable
+            Name of errorbar method (either "ci", "pi", "se", or "sd"), or a tuple
+            with a method name and a level parameter, or a function that maps from a
+            vector to a (min, max) interval.
+        boot_kws
+            Additional keywords are passed to bootstrap when error_method is "ci".
+
+        """
+        self.estimator = estimator
+
+        method, level = _validate_errorbar_arg(errorbar)
+        self.error_method = method
+        self.error_level = level
+
+        self.boot_kws = boot_kws
+
+    def __call__(self, data, var):
+        """Aggregate over `var` column of `data` with estimate and error interval."""
+        vals = data[var]
+        if callable(self.estimator):
+            # You would think we could pass to vals.agg, and yet:
+            # https://github.com/mwaskom/seaborn/issues/2943
+            estimate = self.estimator(vals)
+        else:
+            estimate = vals.agg(self.estimator)
+
+        # Options that produce no error bars
+        if self.error_method is None:
+            err_min = err_max = np.nan
+        elif len(data) <= 1:
+            err_min = err_max = np.nan
+
+        # Generic errorbars from user-supplied function
+        elif callable(self.error_method):
+            err_min, err_max = self.error_method(vals)
+
+        # Parametric options
+        elif self.error_method == "sd":
+            half_interval = vals.std() * self.error_level
+            err_min, err_max = estimate - half_interval, estimate + half_interval
+        elif self.error_method == "se":
+            half_interval = vals.sem() * self.error_level
+            err_min, err_max = estimate - half_interval, estimate + half_interval
+
+        # Nonparametric options
+        elif self.error_method == "pi":
+            err_min, err_max = _percentile_interval(vals, self.error_level)
+        elif self.error_method == "ci":
+            units = data.get("units", None)
+            boots = bootstrap(vals, units=units, func=self.estimator, **self.boot_kws)
+            err_min, err_max = _percentile_interval(boots, self.error_level)
+
+        return pd.Series({var: estimate, f"{var}min": err_min, f"{var}max": err_max})
+
+
+def _percentile_interval(data, width):
+    """Return a percentile interval from data of a given width."""
+    edge = (100 - width) / 2
+    percentiles = edge, 100 - edge
+    return np.nanpercentile(data, percentiles)
+
+
+def _validate_errorbar_arg(arg):
+    """Check type and value of errorbar argument and assign default level."""
+    DEFAULT_LEVELS = {
+        "ci": 95,
+        "pi": 95,
+        "se": 1,
+        "sd": 1,
+    }
+
+    usage = "`errorbar` must be a callable, string, or (string, number) tuple"
+
+    if arg is None:
+        return None, None
+    elif callable(arg):
+        return arg, None
+    elif isinstance(arg, str):
+        method = arg
+        level = DEFAULT_LEVELS.get(method, None)
+    else:
+        try:
+            method, level = arg
+        except (ValueError, TypeError) as err:
+            raise err.__class__(usage) from err
+
+    _check_argument("errorbar", list(DEFAULT_LEVELS), method)
+    if level is not None and not isinstance(level, Number):
+        raise TypeError(usage)
+
+    return method, level
