@@ -1,25 +1,26 @@
+from __future__ import annotations
 from itertools import product
 from inspect import signature
 import warnings
 from textwrap import dedent
-from distutils.version import LooseVersion
 
 import numpy as np
 import pandas as pd
 import matplotlib as mpl
 import matplotlib.pyplot as plt
 
-from ._core import VectorPlotter, variable_type, categorical_order
+from ._oldcore import VectorPlotter, variable_type, categorical_order
+from ._compat import share_axis
 from . import utils
-from .utils import _check_argument, adjust_legend_subtitles, _draw_figure
+from .utils import (
+    adjust_legend_subtitles, _check_argument, _draw_figure, _disable_autolayout
+)
 from .palettes import color_palette, blend_palette
-from ._decorators import _deprecate_positional_args
 from ._docstrings import (
     DocstringComponents,
     _core_docs,
 )
 
-
 __all__ = ["FacetGrid", "PairGrid", "JointGrid", "pairplot", "jointplot"]
 
 
@@ -52,6 +53,35 @@ class _BaseGrid:
         """Access the :class:`matplotlib.figure.Figure` object underlying the grid."""
         return self._figure
 
+    def apply(self, func, *args, **kwargs):
+        """
+        Pass the grid to a user-supplied function and return self.
+
+        The `func` must accept an object of this type for its first
+        positional argument. Additional arguments are passed through.
+        The return value of `func` is ignored; this method returns self.
+        See the `pipe` method if you want the return value.
+
+        Added in v0.12.0.
+
+        """
+        func(self, *args, **kwargs)
+        return self
+
+    def pipe(self, func, *args, **kwargs):
+        """
+        Pass the grid to a user-supplied function and return its value.
+
+        The `func` must accept an object of this type for its first
+        positional argument. Additional arguments are passed through.
+        The return value of `func` becomes the return value of this method.
+        See the `apply` method if you want to return self instead.
+
+        Added in v0.12.0.
+
+        """
+        return func(self, *args, **kwargs)
+
     def savefig(self, *args, **kwargs):
         """
         Save an image of the plot.
@@ -86,6 +116,7 @@ class Grid(_BaseGrid):
         if self._tight_layout_pad is not None:
             kwargs.setdefault("pad", self._tight_layout_pad)
         self._figure.tight_layout(*args, **kwargs)
+        return self
 
     def add_legend(self, legend_data=None, title=None, label_order=None,
                    adjust_subtitles=False, **kwargs):
@@ -127,13 +158,7 @@ class Grid(_BaseGrid):
         blank_handle = mpl.patches.Patch(alpha=0, linewidth=0)
         handles = [legend_data.get(l, blank_handle) for l in label_order]
         title = self._hue_var if title is None else title
-        if LooseVersion(mpl.__version__) < LooseVersion("3.0"):
-            try:
-                title_size = mpl.rcParams["axes.labelsize"] * .85
-            except TypeError:  # labelsize is something like "large"
-                title_size = mpl.rcParams["axes.labelsize"]
-        else:
-            title_size = mpl.rcParams["legend.title_fontsize"]
+        title_size = mpl.rcParams["legend.title_fontsize"]
 
         # Unpack nested labels from a hierarchical legend
         labels = []
@@ -254,6 +279,27 @@ class Grid(_BaseGrid):
         except AttributeError:
             return None
 
+    def tick_params(self, axis='both', **kwargs):
+        """Modify the ticks, tick labels, and gridlines.
+
+        Parameters
+        ----------
+        axis : {'x', 'y', 'both'}
+            The axis on which to apply the formatting.
+        kwargs : keyword arguments
+            Additional keyword arguments to pass to
+            :meth:`matplotlib.axes.Axes.tick_params`.
+
+        Returns
+        -------
+        self : Grid instance
+            Returns self for easy chaining.
+
+        """
+        for ax in self.figure.axes:
+            ax.tick_params(axis=axis, **kwargs)
+        return self
+
 
 _facet_docs = dict(
 
@@ -316,7 +362,7 @@ _facet_docs = dict(
 
 class FacetGrid(Grid):
     """Multi-plot grid for plotting conditional relationships."""
-    @_deprecate_positional_args
+
     def __init__(
         self, data, *,
         row=None, col=None, hue=None, col_wrap=None,
@@ -324,17 +370,10 @@ class FacetGrid(Grid):
         row_order=None, col_order=None, hue_order=None, hue_kws=None,
         dropna=False, legend_out=True, despine=True,
         margin_titles=False, xlim=None, ylim=None, subplot_kws=None,
-        gridspec_kws=None, size=None
+        gridspec_kws=None,
     ):
 
-        super(FacetGrid, self).__init__()
-
-        # Handle deprecations
-        if size is not None:
-            height = size
-            msg = ("The `size` parameter has been renamed to `height`; "
-                   "please update your code.")
-            warnings.warn(msg, UserWarning)
+        super().__init__()
 
         # Determine the hue facet layer information
         hue_var = hue
@@ -404,8 +443,7 @@ class FacetGrid(Grid):
 
         # --- Initialize the subplot grid
 
-        # Disable autolayout so legend_out works properly
-        with mpl.rc_context({"figure.autolayout": False}):
+        with _disable_autolayout():
             fig = plt.figure(figsize=figsize)
 
         if col_wrap is None:
@@ -477,12 +515,16 @@ class FacetGrid(Grid):
         self._legend_data = {}
         self._x_var = None
         self._y_var = None
+        self._sharex = sharex
+        self._sharey = sharey
         self._dropna = dropna
         self._not_na = not_na
 
         # --- Make the axes look good
 
+        self.set_titles()
         self.tight_layout()
+
         if despine:
             self.despine()
 
@@ -527,7 +569,7 @@ class FacetGrid(Grid):
 
             When using seaborn functions that infer semantic mappings from a
             dataset, care must be taken to synchronize those mappings across
-            facets (e.g., by defing the ``hue`` mapping with a palette dict or
+            facets (e.g., by defining the ``hue`` mapping with a palette dict or
             setting the data type of the variables to ``category``). In most cases,
             it will be better to use a figure-level function (e.g. :func:`relplot`
             or :func:`catplot`) than to use :class:`FacetGrid` directly.
@@ -811,7 +853,6 @@ class FacetGrid(Grid):
     def _finalize_grid(self, axlabels):
         """Finalize the annotations and layout."""
         self.set_axis_labels(*axlabels)
-        self.set_titles()
         self.tight_layout()
 
     def facet_axis(self, row_i, col_j, modify_state=True):
@@ -1018,6 +1059,8 @@ class FacetGrid(Grid):
         if y is not None:
             self.map(plt.axhline, y=y, **line_kws)
 
+        return self
+
     # ------ Properties that are part of the public API and documented by Sphinx
 
     @property
@@ -1132,7 +1175,7 @@ class PairGrid(Grid):
     This object maps each variable in a dataset onto a column and row in a
     grid of multiple axes. Different axes-level plotting functions can be
     used to draw bivariate plots in the upper and lower triangles, and the
-    the marginal distribution of each variable can be shown on the diagonal.
+    marginal distribution of each variable can be shown on the diagonal.
 
     Several different common plots can be generated in a single line using
     :func:`pairplot`. Use :class:`PairGrid` when you need more flexibility.
@@ -1140,13 +1183,10 @@ class PairGrid(Grid):
     See the :ref:`tutorial <grid_tutorial>` for more information.
 
     """
-    @_deprecate_positional_args
     def __init__(
-        self, data, *,
-        hue=None, hue_order=None, palette=None,
-        hue_kws=None, vars=None, x_vars=None, y_vars=None,
-        corner=False, diag_sharey=True, height=2.5, aspect=1,
-        layout_pad=.5, despine=True, dropna=False, size=None
+        self, data, *, hue=None, vars=None, x_vars=None, y_vars=None,
+        hue_order=None, palette=None, hue_kws=None, corner=False, diag_sharey=True,
+        height=2.5, aspect=1, layout_pad=.5, despine=True, dropna=False,
     ):
         """Initialize the plot figure and PairGrid object.
 
@@ -1158,6 +1198,12 @@ class PairGrid(Grid):
         hue : string (variable name)
             Variable in ``data`` to map plot aspects to different colors. This
             variable will be excluded from the default x and y variables.
+        vars : list of variable names
+            Variables within ``data`` to use, otherwise use every column with
+            a numeric datatype.
+        {x, y}_vars : lists of variable names
+            Variables within ``data`` to use separately for the rows and
+            columns of the figure; i.e. to make a non-square plot.
         hue_order : list of strings
             Order for the levels of the hue variable in the palette
         palette : dict or seaborn color palette
@@ -1167,12 +1213,6 @@ class PairGrid(Grid):
             Other keyword arguments to insert into the plotting call to let
             other plot attributes vary across levels of the hue variable (e.g.
             the markers in a scatterplot).
-        vars : list of variable names
-            Variables within ``data`` to use, otherwise use every column with
-            a numeric datatype.
-        {x, y}_vars : lists of variable names
-            Variables within ``data`` to use separately for the rows and
-            columns of the figure; i.e. to make a non-square plot.
         corner : bool
             If True, don't add axes to the upper (off-diagonal) triangle of the
             grid, making this a "corner" plot.
@@ -1199,14 +1239,7 @@ class PairGrid(Grid):
 
         """
 
-        super(PairGrid, self).__init__()
-
-        # Handle deprecations
-        if size is not None:
-            height = size
-            msg = ("The `size` parameter has been renamed to `height`; "
-                   "please update your code.")
-            warnings.warn(UserWarning(msg))
+        super().__init__()
 
         # Sort out the variables that define the grid
         numeric_cols = self._find_numeric_cols(data)
@@ -1237,8 +1270,7 @@ class PairGrid(Grid):
         # Create the figure and the array of subplots
         figsize = len(x_vars) * height * aspect, len(y_vars) * height
 
-        # Disable autolayout so legend_out works
-        with mpl.rc_context({"figure.autolayout": False}):
+        with _disable_autolayout():
             fig = plt.figure(figsize=figsize)
 
         axes = fig.subplots(len(y_vars), len(x_vars),
@@ -1437,11 +1469,8 @@ class PairGrid(Grid):
                             # when drawing a corner plot?
 
             if self.diag_sharey and diag_axes:
-                # This may change in future matplotlibs
-                # See https://github.com/matplotlib/matplotlib/pull/9923
-                group = diag_axes[0].get_shared_y_axes()
                 for ax in diag_axes[1:]:
-                    group.join(ax, diag_axes[0])
+                    share_axis(diag_axes[0], ax, "y")
 
             self.diag_vars = np.array(diag_vars, np.object_)
             self.diag_axes = np.array(diag_axes, np.object_)
@@ -1573,9 +1602,10 @@ class PairGrid(Grid):
         else:
             hue = data.get(self._hue_var)
 
-        kwargs.setdefault("hue", hue)
-        kwargs.setdefault("hue_order", self._hue_order)
-        kwargs.setdefault("palette", self._orig_palette)
+        if "hue" not in kwargs:
+            kwargs.update({
+                "hue": hue, "hue_order": self._hue_order, "palette": self._orig_palette,
+            })
         func(x=x, y=y, **kwargs)
 
         self._update_legend_data(ax)
@@ -1630,8 +1660,6 @@ class PairGrid(Grid):
             ax.set_xlabel(label)
         for ax, label in zip(self.axes[:, 0], self.y_vars):
             ax.set_ylabel(label)
-        if self._corner:
-            self.axes[0, 0].set_ylabel("")
 
     def _find_numeric_cols(self, data):
         """Find which variables in a DataFrame are numeric."""
@@ -1650,21 +1678,13 @@ class JointGrid(_BaseGrid):
 
     """
 
-    @_deprecate_positional_args
     def __init__(
-        self, *,
-        x=None, y=None,
-        data=None,
+        self, data=None, *,
+        x=None, y=None, hue=None,
         height=6, ratio=5, space=.2,
-        dropna=False, xlim=None, ylim=None, size=None, marginal_ticks=False,
-        hue=None, palette=None, hue_order=None, hue_norm=None,
+        palette=None, hue_order=None, hue_norm=None,
+        dropna=False, xlim=None, ylim=None, marginal_ticks=False,
     ):
-        # Handle deprecations
-        if size is not None:
-            height = size
-            msg = ("The `size` parameter has been renamed to `height`; "
-                   "please update your code.")
-            warnings.warn(msg, UserWarning)
 
         # Set up the subplot grid
         f = plt.figure(figsize=(height, height))
@@ -1943,8 +1963,8 @@ Set up the grid of subplots and store data internally for easy plotting.
 
 Parameters
 ----------
-{params.core.xy}
 {params.core.data}
+{params.core.xy}
 height : number
     Size of each side of the figure in inches (it will be square).
 ratio : number
@@ -1982,7 +2002,6 @@ Examples
 )
 
 
-@_deprecate_positional_args
 def pairplot(
     data, *,
     hue=None, hue_order=None, palette=None,
@@ -2077,8 +2096,7 @@ def pairplot(
 
     if not isinstance(data, pd.DataFrame):
         raise TypeError(
-            "'data' must be pandas DataFrame object, not: {typefound}".format(
-                typefound=type(data)))
+            f"'data' must be pandas DataFrame object, not: {type(data)}")
 
     plot_kws = {} if plot_kws is None else plot_kws.copy()
     diag_kws = {} if diag_kws is None else diag_kws.copy()
@@ -2109,8 +2127,8 @@ def pairplot(
             if not isinstance(markers, list):
                 markers = [markers] * n_markers
             if len(markers) != n_markers:
-                raise ValueError(("markers must be a singleton or a list of "
-                                  "markers for each level of the hue variable"))
+                raise ValueError("markers must be a singleton or a list of "
+                                 "markers for each level of the hue variable")
             grid.hue_kws = {"marker": markers}
         elif kind == "scatter":
             if isinstance(markers, str):
@@ -2158,15 +2176,11 @@ def pairplot(
     return grid
 
 
-@_deprecate_positional_args
 def jointplot(
-    *,
-    x=None, y=None,
-    data=None,
-    kind="scatter", color=None, height=6, ratio=5, space=.2,
-    dropna=False, xlim=None, ylim=None, marginal_ticks=False,
+    data=None, *, x=None, y=None, hue=None, kind="scatter",
+    height=6, ratio=5, space=.2, dropna=False, xlim=None, ylim=None,
+    color=None, palette=None, hue_order=None, hue_norm=None, marginal_ticks=False,
     joint_kws=None, marginal_kws=None,
-    hue=None, palette=None, hue_order=None, hue_norm=None,
     **kwargs
 ):
     # Avoid circular imports
@@ -2174,12 +2188,9 @@ def jointplot(
     from .regression import regplot, residplot
     from .distributions import histplot, kdeplot, _freedman_diaconis_bins
 
-    # Handle deprecations
-    if "size" in kwargs:
-        height = kwargs.pop("size")
-        msg = ("The `size` parameter has been renamed to `height`; "
-               "please update your code.")
-        warnings.warn(msg, UserWarning)
+    if kwargs.pop("ax", None) is not None:
+        msg = "Ignoring `ax`; jointplot is a figure-level function."
+        warnings.warn(msg, UserWarning, stacklevel=2)
 
     # Set up empty default kwarg dicts
     joint_kws = {} if joint_kws is None else joint_kws.copy()
@@ -2256,7 +2267,7 @@ def jointplot(
     elif kind.startswith("hist"):
 
         # TODO process pair parameters for bins, etc. and pass
-        # to both jount and marginal plots
+        # to both joint and marginal plots
 
         joint_kws.setdefault("color", color)
         grid.plot_joint(histplot, **joint_kws)
@@ -2322,6 +2333,9 @@ def jointplot(
         histplot(x=x, hue=hue, ax=grid.ax_marg_x, **marginal_kws)
         histplot(y=y, hue=hue, ax=grid.ax_marg_y, **marginal_kws)
 
+    # Make the main axes active in the matplotlib state machine
+    plt.sca(grid.ax_joint)
+
     return grid
 
 
@@ -2335,11 +2349,12 @@ lightweight wrapper; if you need more flexibility, you should use
 
 Parameters
 ----------
-{params.core.xy}
 {params.core.data}
+{params.core.xy}
+{params.core.hue}
+    Semantic variable that is mapped to determine the color of plot elements.
 kind : {{ "scatter" | "kde" | "hist" | "hex" | "reg" | "resid" }}
     Kind of plot to draw. See the examples for references to the underlying functions.
-{params.core.color}
 height : numeric
     Size of the figure (it will be square).
 ratio : numeric
@@ -2350,15 +2365,14 @@ dropna : bool
     If True, remove observations that are missing from ``x`` and ``y``.
 {{x, y}}lim : pairs of numbers
     Axis limits to set before plotting.
+{params.core.color}
+{params.core.palette}
+{params.core.hue_order}
+{params.core.hue_norm}
 marginal_ticks : bool
     If False, suppress ticks on the count/density axis of the marginal plots.
 {{joint, marginal}}_kws : dicts
     Additional keyword arguments for the plot components.
-{params.core.hue}
-    Semantic variable that is mapped to determine the color of plot elements.
-{params.core.palette}
-{params.core.hue_order}
-{params.core.hue_norm}
 kwargs
     Additional keyword arguments are passed to the function used to
     draw the plot on the joint Axes, superseding items in the
