@@ -7,10 +7,15 @@ import matplotlib.pyplot as plt
 from matplotlib import gridspec
 import numpy as np
 import pandas as pd
-from scipy.cluster import hierarchy
+try:
+    from scipy.cluster import hierarchy
+    _no_scipy = False
+except ImportError:
+    _no_scipy = True
 
 from . import cm
 from .axisgrid import Grid
+from ._compat import get_colormap
 from .utils import (
     despine,
     axis_ticklabels_overlap,
@@ -18,7 +23,6 @@ from .utils import (
     to_utf8,
     _draw_figure,
 )
-from ._decorators import _deprecate_positional_args
 
 
 __all__ = ["heatmap", "clustermap"]
@@ -105,7 +109,7 @@ class _HeatMapper:
             plot_data = np.asarray(data)
             data = pd.DataFrame(plot_data)
 
-        # Validate the mask and convet to DataFrame
+        # Validate the mask and convert to DataFrame
         mask = _matrix_mask(data, mask)
 
         plot_data = np.ma.masked_where(np.asarray(mask), plot_data)
@@ -210,7 +214,7 @@ class _HeatMapper:
             else:
                 self.cmap = cm.icefire
         elif isinstance(cmap, str):
-            self.cmap = mpl.cm.get_cmap(cmap)
+            self.cmap = get_colormap(cmap)
         elif isinstance(cmap, list):
             self.cmap = mpl.colors.ListedColormap(cmap)
         else:
@@ -348,7 +352,6 @@ class _HeatMapper:
             self._annotate_heatmap(ax, mesh)
 
 
-@_deprecate_positional_args
 def heatmap(
     data, *,
     vmin=None, vmax=None, cmap=None, center=None, robust=False,
@@ -379,7 +382,7 @@ def heatmap(
         The mapping from data values to color space. If not provided, the
         default will depend on whether ``center`` is set.
     center : float, optional
-        The value at which to center the colormap when plotting divergant data.
+        The value at which to center the colormap when plotting divergent data.
         Using this parameter will change the default ``cmap`` if none is
         specified.
     robust : bool, optional
@@ -430,7 +433,7 @@ def heatmap(
 
     See Also
     --------
-    clustermap : Plot a matrix using hierachical clustering to arrange the
+    clustermap : Plot a matrix using hierarchical clustering to arrange the
                  rows and columns.
 
     Examples
@@ -554,7 +557,7 @@ def heatmap(
     return ax
 
 
-class _DendrogramPlotter(object):
+class _DendrogramPlotter:
     """Object for drawing tree of similarities between data rows/columns"""
 
     def __init__(self, data, linkage, metric, method, axis, label, rotate):
@@ -733,7 +736,6 @@ class _DendrogramPlotter(object):
         return self
 
 
-@_deprecate_positional_args
 def dendrogram(
     data, *,
     linkage=None, axis=1, label=True, metric='euclidean',
@@ -776,6 +778,9 @@ def dendrogram(
     dendrogramplotter.reordered_ind
 
     """
+    if _no_scipy:
+        raise RuntimeError("dendrogram requires scipy to be installed")
+
     plotter = _DendrogramPlotter(data, linkage=linkage, axis=axis,
                                  metric=metric, method=method,
                                  label=label, rotate=rotate)
@@ -791,6 +796,8 @@ class ClusterGrid(Grid):
                  figsize=None, row_colors=None, col_colors=None, mask=None,
                  dendrogram_ratio=None, colors_ratio=None, cbar_pos=None):
         """Grid object for organizing clustered heatmap input on to axes"""
+        if _no_scipy:
+            raise RuntimeError("ClusterGrid requires scipy to be available")
 
         if isinstance(data, pd.DataFrame):
             self.data = data
@@ -992,7 +999,7 @@ class ClusterGrid(Grid):
         ratios = [dendrogram_ratio]
 
         if colors is not None:
-            # Colors are encoded as rgb, so ther is an extra dimention
+            # Colors are encoded as rgb, so there is an extra dimension
             if np.ndim(colors) > 2:
                 n_colors = len(colors)
             else:
@@ -1233,7 +1240,6 @@ class ClusterGrid(Grid):
         return self
 
 
-@_deprecate_positional_args
 def clustermap(
     data, *,
     pivot_kws=None, method='average', metric='euclidean',
@@ -1248,6 +1254,8 @@ def clustermap(
     """
     Plot a matrix dataset as a hierarchically-clustered heatmap.
 
+    This function requires scipy to be available.
+
     Parameters
     ----------
     data : 2D array-like
@@ -1397,6 +1405,9 @@ def clustermap(
 
         >>> g = sns.clustermap(iris, z_score=0, cmap="vlag")
     """
+    if _no_scipy:
+        raise RuntimeError("clustermap requires scipy to be available")
+
     plotter = ClusterGrid(data, pivot_kws=pivot_kws, figsize=figsize,
                           row_colors=row_colors, col_colors=col_colors,
                           z_score=z_score, standard_scale=standard_scale,
