@@ -8,6 +8,7 @@ from .external import husl
 
 from .utils import desaturate, get_color_cycle
 from .colors import xkcd_rgb, crayons
+from ._compat import get_colormap
 
 
 __all__ = ["color_palette", "hls_palette", "husl_palette", "mpl_palette",
@@ -111,7 +112,7 @@ def color_palette(palette=None, n_colors=None, desat=None, as_cmap=False):
 
     Parameters
     ----------
-    palette: None, string, or sequence, optional
+    palette : None, string, or sequence, optional
         Name of palette or None to return current palette. If a sequence, input
         colors are used but possibly cycled and desaturated.
     n_colors : int, optional
@@ -204,8 +205,8 @@ def color_palette(palette=None, n_colors=None, desat=None, as_cmap=False):
             try:
                 # Perhaps a named matplotlib colormap?
                 palette = mpl_palette(palette, n_colors, as_cmap=as_cmap)
-            except ValueError:
-                raise ValueError("%s is not a valid palette name" % palette)
+            except (ValueError, KeyError):  # Error class changed in mpl36
+                raise ValueError(f"{palette} is not a valid palette name")
 
     if desat is not None:
         palette = [desaturate(c, desat) for c in palette]
@@ -440,7 +441,7 @@ def mpl_palette(name, n_colors=6, as_cmap=False):
             pal = pal[::-1]
         cmap = blend_palette(pal, n_colors, as_cmap=True)
     else:
-        cmap = mpl.cm.get_cmap(name)
+        cmap = get_colormap(name)
 
     if name in MPL_QUAL_PALS:
         bins = np.linspace(0, 1, MPL_QUAL_PALS[name])[:n_colors]
@@ -1029,7 +1030,7 @@ def set_color_codes(palette="deep"):
             palette = palette + "6"
         colors = SEABORN_PALETTES[palette] + [(.1, .1, .1)]
     else:
-        err = "Cannot set colors with palette '{}'".format(palette)
+        err = f"Cannot set colors with palette '{palette}'"
         raise ValueError(err)
 
     for code, color in zip("bgrmyck", colors):
